"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const MapNameToId_1 = require("../generators/dot/utils/MapNameToId");
const Edge_1 = require("./Edge");
const GraphCluster_1 = require("./GraphCluster");
const GraphNode_1 = require("./GraphNode");
const MapIdToGraphNode_1 = require("./utils/MapIdToGraphNode");
/**
 * Generate a graph that represents the structure described in the tslint config.
 *
 * The graph can then be optimized before rendering (mainly to reduce the number of edges).
 *
 * note: the graph could contain circular references!
 */
class GraphGenerator {
    constructor(config, filter) {
        this.config = config;
        this.filter = filter;
        // the root cluster is 0 (root is not output)
        this.containerId = 0;
        this.mapNameToId = new MapNameToId_1.MapNameToId();
        this.mapIdToNode = new MapIdToGraphNode_1.MapIdToGraphNode();
        this.root = GraphCluster_1.GraphCluster.create(null, this.containerId++, "root", "", GraphCluster_1.ClusterType.Root);
        this.mapIdToNode.add(this.root);
    }
    generateGraph(packageConfig) {
        const packageFolders = packageConfig.checkImportsBetweenPackages.packages.filter(pkg => this.filter.isImportPathOkForFolder(pkg));
        this.checkIfNeedsAnyPackage(packageFolders);
        let parent = this.root;
        const topLevelCluster = GraphCluster_1.GraphCluster.create(this.root, this.containerId++, "Top Level Packages", "", GraphCluster_1.ClusterType.TopLevel);
        if (!this.filter.hasWhitelist) {
            this.root.nodes.push(topLevelCluster);
        }
        parent = topLevelCluster;
        parent.nodes.push(...this.generateNodesFromPackages(parent, packageFolders));
        packageFolders.forEach(pkg => {
            this.processPackageSubFolders(this.root, pkg);
        });
        this.processEdges(packageFolders.filter(pkg => this.filter.canOutputSubfoldersOf(pkg)));
        return this.root;
    }
    checkIfNeedsAnyPackage(packageFolders) {
        if (!this.config.dot.showImportAnyAsNodeNotEdges) {
            return;
        }
        if (packageFolders.some(pkg => pkg.allowedToImport.some(allowed => allowed === "*"))) {
            packageFolders.push(this.createAnyPackage());
        }
        // also for sub-folders:
        packageFolders.forEach(pkg => {
            if (pkg.subFolders.some(sub => sub.allowedToImport.some(allowed => allowed === "*"))) {
                pkg.subFolders.push(this.createAnySubPackage());
            }
        });
    }
    generateNodesFromPackages(cluster, packageFolders) {
        if (!this.config.dot.clusterFromTslintJson) {
            return packageFolders.map(pkg => this.generateNodeFromPackage(cluster, pkg));
        }
        const defaultCluster = "(defaultCluster)";
        const packagesByCluster = lodash_1.groupBy(packageFolders, (pkg) => pkg.diagramCluster || defaultCluster);
        const nodes = [];
        Object.keys(packagesByCluster).forEach(key => {
            const pkgs = packagesByCluster[key];
            const nodesThisCluster = pkgs.map(pkg => this.generateNodeFromPackage(cluster, pkg));
            if (key === defaultCluster) {
                nodes.push(...nodesThisCluster);
            }
            if (key !== defaultCluster) {
                const subCluster = GraphCluster_1.GraphCluster.create(cluster, this.containerId++, "", "", GraphCluster_1.ClusterType.DiagramCluster);
                this.mapIdToNode.add(cluster);
                nodes.push(subCluster);
                subCluster.nodes.push(...nodesThisCluster);
                nodesThisCluster.forEach(n => (n.parent = subCluster));
            }
        });
        return nodes;
    }
    generateNodeFromPackage(cluster, pkg) {
        const packageName = pkg.importPath;
        const node = this.generateNode(cluster, packageName, pkg.description, this.getNodeType(pkg));
        return node;
    }
    getNodeType(pkg) {
        if (pkg.isExternal) {
            return GraphNode_1.NodeType.External;
        }
        if (pkg.importPath === this.getAnyPackageId()) {
            return GraphNode_1.NodeType.Any;
        }
        return GraphNode_1.NodeType.Normal;
    }
    getNodeTypeForSubFolder(pkg) {
        if (pkg.importPath === this.getAnyPackageId()) {
            return GraphNode_1.NodeType.Any;
        }
        return GraphNode_1.NodeType.Normal;
    }
    /**
     * process the sub-folders of a package as a separate graph cluster.
     *
     * Else, the graph is too hard to read.
     */
    processPackageSubFolders(parent, pkg) {
        const isContainer = pkg.subFolders.length > 0;
        if (!isContainer) {
            return;
        }
        if (!this.filter.canOutputSubfoldersOf(pkg)) {
            return;
        }
        const cluster = GraphCluster_1.GraphCluster.create(parent, this.containerId++, pkg.importPath, pkg.description, GraphCluster_1.ClusterType.AreaWithSubFolders);
        this.mapIdToNode.add(cluster);
        const nodes = this.generateSubFolders(cluster, pkg.importPath, pkg.subFolders.filter(sub => this.filter.isImportPathOkForSubFolder(sub)));
        cluster.nodes.push(...nodes);
        this.root.nodes.push(cluster);
    }
    generateNode(cluster, packageName, description, nodeType = GraphNode_1.NodeType.Normal, prefix) {
        const packageIdKey = this.getPackageIdKey(packageName, prefix);
        const packageId = this.mapNameToId.getId(packageIdKey);
        const node = new GraphNode_1.GraphNode(cluster, packageId, packageName, description, nodeType);
        this.mapIdToNode.add(node);
        return node;
    }
    getPackageIdKey(packageName, prefix) {
        return `${prefix}_${packageName}`;
    }
    processEdges(packageFolders) {
        packageFolders.forEach(pkg => {
            const thisPkgId = this.mapNameToId.getIdOrThrow(this.getPackageIdKey(pkg.importPath));
            let allowedPackages = pkg.allowedToImport;
            if (allowedPackages.some(allowed => allowed === "*")) {
                if (this.config.dot.showImportAnyAsNodeNotEdges) {
                    allowedPackages = [this.getAnyPackageId()];
                }
                else {
                    allowedPackages = packageFolders
                        // disallow import from self
                        .filter(allowed => allowed.importPath !== pkg.importPath)
                        .map(allowed => allowed.importPath);
                }
            }
            this.processEdgesForPackageNames(thisPkgId, allowedPackages);
            // TODO review - kind of duplicate code - could have interface across PackageFolder, PackageSubFolder ?
            pkg.subFolders
                .filter(sub => this.filter.isImportPathOkForSubFolder(sub))
                .forEach(subFolder => {
                const thisSubFolderId = this.mapNameToId.getId(this.getPackageIdKey(subFolder.importPath, pkg.importPath));
                let allowedSubFolders = subFolder.allowedToImport;
                if (allowedSubFolders.some(allowed => allowed === "*")) {
                    if (this.config.dot.showImportAnyAsNodeNotEdges) {
                        allowedSubFolders = [this.getAnyPackageId()];
                    }
                    else {
                        allowedSubFolders = pkg.subFolders
                            .filter(
                        // disallow import from self
                        otherSubFolder => otherSubFolder.importPath !== subFolder.importPath)
                            .map(sub => sub.importPath);
                    }
                }
                this.processEdgesForPackageNames(thisSubFolderId, allowedSubFolders, pkg.importPath);
            });
        });
    }
    getAnyPackageId() {
        return "*";
    }
    createAnyPackage() {
        return {
            allowedToImport: [],
            description: "(can import any packages)",
            importPath: this.getAnyPackageId(),
            isExternal: false,
            subFolders: []
        };
    }
    createAnySubPackage() {
        return {
            allowedToImport: [],
            description: "(can import any folders)",
            importPath: this.getAnyPackageId()
        };
    }
    processEdgesForPackageNames(thisPkgId, allowedPackages, packageIdPrefix) {
        allowedPackages
            .filter(pkg => this.filter.isImportPathOkForEdges(pkg))
            .forEach(allowedPkg => {
            const allowedPkgId = this.mapNameToId.getIdOrThrow(this.getPackageIdKey(allowedPkg, packageIdPrefix));
            const origin = this.mapIdToNode.getByIdOrThrow(thisPkgId);
            const destination = this.mapIdToNode.getByIdOrThrow(allowedPkgId);
            Edge_1.Edge.create(origin, destination);
        });
    }
    generateSubFolders(cluster, packageName, subFolders) {
        if (subFolders.length === 0) {
            return [];
        }
        return subFolders.map(folder => this.generateNode(cluster, folder.importPath, folder.description, this.getNodeTypeForSubFolder(folder), packageName));
    }
}
exports.GraphGenerator = GraphGenerator;
//# sourceMappingURL=GraphGenerator.js.map